/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;



import java.awt.Graphics2D;



import org.gfxtest.callbacks.SlopedLineCallbacks;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test suite checks rendering of various lines with various slopes.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.PRINT_TEST)
@GraphicsPrimitive(GraphicsPrimitives.LINE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestSlopedLines extends PrintTest
{
    /**
     * Method which renders set of sloped lines using various colors and
     * stroke styles. For each line, the callback function/method is called to
     * perform all required setup.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param slopedLineCallbacks 
     *            class containing set of callback methods
     */
    private void drawSlopedLinesMethod1(TestImage image, Graphics2D graphics, SlopedLineCallbacks slopedLineCallbacks)
    {
        // setup rendering
        slopedLineCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // center of image
        final int xc = width / 2;
        final int yc = height / 2;

        // other parameters
        final int maxRadius = Math.min(width / 2 - BORDER, height / 2 - BORDER);

        // index to color palette
        int colorIndex = 0;

        // draw all lines in a loop
        for (float angle = 0.0f; angle < 2.0f * Math.PI; angle += Math.PI / 16.0f)
        {
            // setup can be made for each line
            slopedLineCallbacks.iterationCallBack(angle, colorIndex++);
            int dx = (int) Math.round(maxRadius * Math.cos(angle));
            int dy = (int) Math.round(maxRadius * Math.sin(angle));
            // render the line
            graphics.drawLine(xc, yc, xc + dx, yc + dy);
        }

        // cleanup rendering
        slopedLineCallbacks.cleanup();
    }

    /**
     * Method which renders set of sloped lines using various colors and stroke
     * styles. For each line, the callback function/method is called to perform
     * all required setup.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param slopedLineCallbacks
     *            class containing set of callback methods
     */
    private void drawSlopedLinesMethod2(TestImage image, Graphics2D graphics, SlopedLineCallbacks slopedLineCallbacks)
    {
        // setup rendering
        slopedLineCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // center of image
        final int xc = width / 2;
        final int yc = height / 2;

        // other parameters
        final int maxRadius = Math.min(width / 2 - BORDER, height / 2 - BORDER);
        float radius = 20.0f;
        float angle = 0.0f;

        // index to color palette
        int colorIndex = 0;

        // draw all lines in a loop
        while (radius < maxRadius)
        {
            // setup can be made for each line
            slopedLineCallbacks.iterationCallBack(angle, colorIndex++);

            // render the lines
            int dx = (int) Math.round(radius * Math.cos(angle));
            int dy = (int) Math.round(radius * Math.sin(angle));
            graphics.drawLine(xc + dx, yc + dy, xc - dy, yc + dx);
            graphics.drawLine(xc + dx, yc + dy, xc + dy, yc - dx);
            graphics.drawLine(xc - dx, yc - dy, xc + dy, yc - dx);
            graphics.drawLine(xc - dx, yc - dy, xc - dy, yc + dx);
            angle += 5.0f * Math.PI / 360.0;
            radius += 5.0f;
        }

        // cleanup rendering
        slopedLineCallbacks.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.drawLine(). Lines at various
     * slopes are rendered with default width and default end caps. Color of all
     * rendered lines are set to black.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawSlopedLinesBasicStyle1(TestImage image, Graphics2D graphics)
    {
        drawSlopedLinesMethod1(image, graphics, new SlopedLineCallbacks()
        {
            @Override
            public void iterationCallBack(float angle, int index)
            {
                return;
            }
        });
        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawLine(). Lines at various
     * slopes are rendered with default width and default end caps. Color of all
     * rendered lines are set to black.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawSlopedLinesBasicStyle2(TestImage image, Graphics2D graphics)
    {
        drawSlopedLinesMethod2(image, graphics, new SlopedLineCallbacks()
        {
            @Override
            public void iterationCallBack(float angle, int index)
            {
                return;
            }
        });
        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestSlopedLines().runTestSuite(args);
    }

}
